"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startServer = void 0;
const child_process_1 = require("child_process");
const http_1 = require("http");
const path_1 = require("path");
const toUnix_1 = __importDefault(require("./toUnix"));
const userData_1 = require("./userData");
async function startServer(project) {
    project = (0, toUnix_1.default)(project);
    console.log("Starting Phaser Editor 2D Core server");
    const savedPort = userData_1.userData.getProjectPort(project);
    console.log("Saved port: " + savedPort);
    const port = await findFreePort(savedPort ?? 1986 + Math.floor(Math.random() * 1024));
    if (savedPort === undefined) {
        console.log(`Assign new port ${port} to ${project}`);
        userData_1.userData.setProjectPort(project, port);
    }
    const fileName = process.platform === "win32" ? "PhaserEditor2D.exe" : "PhaserEditor2D";
    const filePath = (0, toUnix_1.default)((0, path_1.normalize)((0, path_1.join)((0, toUnix_1.default)(__dirname), `../../server/${fileName}`)));
    console.log(`Spawn: ${filePath}`);
    const args = ["-disable-open-browser", "-port", port.toString(), "-project", project];
    console.log(args);
    let proc;
    try {
        const serverProc = proc = (0, child_process_1.spawn)(filePath, args, {
            windowsHide: true
        });
        console.log(`Process ID: ${serverProc.pid}`);
        serverProc.once("close", (code) => {
            console.log(`Closed Phaser Editor 2D Core server (${serverProc.pid}). Exit code (${code}).`);
        });
        serverProc.stdout?.pipe(process.stdout);
        serverProc.stderr?.pipe(process.stderr);
        process.on("exit", code => {
            console.log(`Main process exit. Kill server proc (${serverProc.pid}). Exit code (${code}).`);
            console.log("Kill server process " + serverProc.pid);
            serverProc.kill("SIGKILL");
        });
    }
    catch (e) {
        console.log(e);
    }
    return { port, proc };
}
exports.startServer = startServer;
async function isFreePort(port) {
    return new Promise(resolve => {
        const server = (0, http_1.createServer)();
        server.listen(port, () => {
            server.close();
            resolve(true);
        });
        server.on("error", () => {
            resolve(false);
        });
    });
}
async function findFreePort(portStart) {
    while (true) {
        if (await isFreePort(portStart)) {
            return portStart;
        }
        portStart++;
    }
}
